package flocking

import java.awt.Graphics
import java.awt.{Point => JPoint}
import java.awt.Graphics2D
import java.awt.RenderingHints
import java.awt.Color
import java.awt.Dimension
import java.awt.Shape
import java.awt.geom.Line2D
import java.awt.Rectangle
import java.awt.geom.AffineTransform
import java.awt.event.ActionListener
import java.awt.event.ActionEvent
import java.awt.Image
import java.awt.image.BufferedImage
import java.awt.GraphicsEnvironment
import java.awt.GraphicsDevice
import java.awt.GraphicsConfiguration
import javax.imageio.ImageIO
import java.io.File
import java.awt.BasicStroke

import javax.swing.Timer

import scala.swing._
import scala.swing.RichWindow._
import scala.swing.event._
import scala.math._

// import flocking.Heading
// import flocking.Model
// import flocking.ModelIterator

trait Visu {
  val model: Model
  val pixelWidth: Int
  val pixelHeight: Int
  val frameDelay: Int
  val birdLength:Double
  val birdWidth:Double
  val gc: GraphicsConfiguration

//  var birdsShapes: Seq[Shape] = List()
  // val birdImage: Image = ImageIO.read(getClass().getResource("/hawk.png")).getScaledInstance(50, 50, Image.SCALE_DEFAULT)
  // val birdImageWidth: Int = birdImage.getWidth(null)
  // val birdImageHeight: Int = birdImage.getHeight(null)
  // val birdImageScale: Double = 0.05
  var backgroundImage: BufferedImage = new BufferedImage(model.env.nCellsWide, model.env.nCellsHigh, BufferedImage.TYPE_INT_RGB)
  val scaleBackgroundImage: AffineTransform = new AffineTransform()
  scaleBackgroundImage.scale(pixelWidth / model.env.nCellsWide.toDouble, pixelHeight / model.env.nCellsHigh.toDouble)
  val backgroundColorRGB:Int = new Color(0,0,0).getRGB()
  val obstacleColorRGB:Int = new Color(0,0,0).getRGB()
  val birdColor: Color = new Color(255,255,255)

  var lastFrameTime: Long = System.currentTimeMillis()
  var framesSinceLastTimeMeasure: Int = 0
  var deltaTime: Long = 0


  def shapeBird(x: Double,y: Double,heading: Heading): Shape = {
    new Line2D.Double(
      (x / model.worldWidth * pixelWidth + (birdLength / 2) * cos(heading.toDouble)),
      (y / model.worldHeight * pixelHeight + (birdLength / 2) * sin(heading.toDouble)),
      (x / model.worldWidth * pixelWidth - (birdLength / 2) * cos(heading.toDouble)),
      (y / model.worldHeight * pixelHeight - (birdLength / 2) * sin(heading.toDouble))
    )
  }

  def drawBirds(xyheading: Seq[(Double, Double, Heading)]) =
    xyheading.map(t => shapeBird(t._1, t._2, t._3))

  def birdsShapes(): Iterable[Shape] =
    drawBirds(modelStepByStep.currentState.birds.map(b => (b.position.x, b.position.y, b.heading)))

  // def birdsTransform(): Iterable[AffineTransform] =
  //   modelStepByStep.currentState.birds.map(b => {
  //       //b.position.x, b.position.y, b.heading
  //       val af = new AffineTransform()
  //       val xpos = b.position.x / model.worldWidth * pixelWidth
  //       val ypos = b.position.y / model.worldHeight * pixelHeight
  //       af.translate(xpos - birdImageWidth / 2.0, ypos - birdImageHeight / 2.0)
  //       // af.rotate(b.heading.toDouble + Pi/2, xpos, ypos)
        
  //       af
  //     }
  //   )

  def updateBackground() {
    val envWidth: Double = model.env.width
    val envHeight: Double = model.env.height
    // var i: Int = 0
    // var j: Int = 0
    // i = 0
    // while (i < pixelWidth) { 
    //   j = 0
    //   while (j < pixelHeight) {
    //     if (model.env.get((i + 0.5) * envWidth / pixelWidth, (j + 0.5) * envHeight / pixelHeight) != 0) backgroundImage.setRGB(i,j,obstacleColorRGB)
    //     else backgroundImage.setRGB(i,j,backgroundColorRGB)
    //     j += 1
    //   }
    //   i += 1
    // }
    backgroundImage.setRGB(0,0, model.env.nCellsWide, model.env.nCellsHigh, model.env.pixels, 0, model.env.nCellsWide)
  }

  val timer = new Timer(frameDelay, Surface)
  val modelStepByStep = new ModelIterator(model)

  object Surface extends Panel with ActionListener  {
    override def paintComponent(g: Graphics2D) = {
      super.paintComponent(g)
      g.setRenderingHint(RenderingHints.KEY_ANTIALIASING, // Anti-alias!
        RenderingHints.VALUE_ANTIALIAS_ON);
      g.setBackground(Color.black)
      g.clearRect(0,0,pixelWidth,pixelHeight)
      g.setColor(birdColor)
      //for {s <- obstaclesShapes} g.fill(s)
      updateBackground
      g.drawImage(backgroundImage, scaleBackgroundImage, null)
      g.setStroke(new BasicStroke(birdWidth.toFloat))
      for {shape <- birdsShapes} g.draw(shape)
      //for {af <- birdsTransform} g.drawImage(birdImage, af, null)
      // val af = new AffineTransform()
      // af.translate(-birdImageWidth / 2.0, -birdImageHeight / 2.0)
      // af.scale(0.5, 0.5)
      // af.translate(birdImageWidth * (1/0.5) / 2.0, birdImageHeight * (1/0.5) / 2.0)
      // af.rotate(Pi/4.0, birdImageWidth / 2.0, birdImageHeight / 2.0)
      // g.drawImage(birdImage, af, null)
      framesSinceLastTimeMeasure += 1
      deltaTime = System.currentTimeMillis() - lastFrameTime
      if (deltaTime >= 1000) {
        print((framesSinceLastTimeMeasure / deltaTime.toDouble) * 1000)
        println(" fps")
        framesSinceLastTimeMeasure = 0
        lastFrameTime = System.currentTimeMillis()
      }
    }

    def actionPerformed(e: ActionEvent) {
      modelStepByStep.step
      // drawEnvironment(modelStepByStep.model.env)
      repaint()
    }

    listenTo(keys)
    reactions += {
      case KeyPressed(_, Key.Q, _, _) => {
        Skeleton.quit()
      }
      case KeyPressed(_, Key.Escape, _, _) => {
        Skeleton.quit()
      }
    }
    focusable = true
    requestFocus
  }


  object Skeleton extends SimpleSwingApplication {
    val f = new MainFrame(gc) with Undecorated {
      preferredSize = new Dimension(pixelWidth, pixelHeight)
      contents = Surface
      background = Color.black
      //location = new JPoint(0,0)
      //maximize
    }

    def top = {
      timer.start()
      f
    }

    override def shutdown() = {
      timer.stop
      super.shutdown
    }
  }
}
//public class Skeleton extends JFrame {
//
//  public Skeleton() {
//
//    initUI();
//  }
//
//  private void initUI() {
//
//    setTitle("Simple Java 2D example");
//
//    add(new Surface());
//
//    setSize(300, 200);
//    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
//    setLocationRelativeTo(null);
//  }
//
//  public static void main(String[] args) {
//
//    SwingUtilities.invokeLater(new Runnable() {
//
//      @Override
//      public void run() {
//
//        Skeleton sk = new Skeleton();
//        sk.setVisible(true);
//      }
//    });
//  }
//}